/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.mojo;

import com.celum.dbtool.installer.*;
import com.celum.dbtool.script.DbScript;
import com.celum.dbtool.script.VersionFactory;
import org.apache.maven.plugin.AbstractMojo;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Map;

/**
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class AbstractDbMojo extends AbstractMojo implements DbEventListener
{
	/**
	 * @parameter
	 */
	protected Map<String, String> parameters;


	/**
	 * @parameter expression="${versionSql}"
	 */
	protected String versionSql;

    /**
     * @parameter
     */
    protected String versionAutoUpdate;

    /**
     * @parameter
     */
    protected String jdbcDriver;

    /**
     * @parameter
     */
    protected String jdbcUrl;


    /**
	 * @parameter
	 */
	protected DbAccount adminAccount;


	/**
	 * @parameter
	 */
	protected DbAccount appAccount;


	/**
	 * @parameter default-value="${basedir}/src/main/sql/create"
	 */
	protected File createScripts;


	/**
	 * @parameter default-value="${basedir}/src/main/sql/scheme"
	 */
	protected File schemeScripts;


	/**
	 * @parameter default-value="${basedir}/src/main/sql/patch"
	 */
	protected File patchScripts;


	/**
	 * @parameter default-value="${basedir}/src/main/sql/drop"
	 */
	protected File dropScripts;


    /**
     * @parameter
     */
    private String versionFactoryClass;


    /**
     * {@inheritDoc}
     */
	@Override
	public DbScript onProcessScript(DbScript script)
	{
		getLog().info("process script:" + script.getName());
		return script;
	}


    /**
     * {@inheritDoc}
     */
    @Override
    public void onError(DbScript script, Throwable e)
    {
        getLog().error("Error in script:" + script.getName(), e);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void onFinishProcessing(DbScript script)
    {
        getLog().info("script:" + script.getName() + " finished OK");
    }


    /**
     * method returns instance of version factory by versionFactoryClass,
     * the default is VersionFactory.DEFAULT.
     *
     * @return instance of version factory
     */
    protected VersionFactory createVersionFactoryFromSettings() throws ClassNotFoundException, IllegalAccessException, InstantiationException
    {
        if (versionFactoryClass == null || versionFactoryClass.isEmpty()) {
            return VersionFactory.DEFAULT;
        }

        Class<?> vfClass = (Class<?>) Class.forName(versionFactoryClass);
        return (VersionFactory) vfClass.newInstance();
    }


    /**
     * if no connection exists, method establish a new DB connection
     * by given user/password, DB driver etc.
     */
    protected Connection getConnection(DbAccount account) throws ClassNotFoundException, SQLException
    {
        Class.forName(jdbcDriver);
        if (account.getJdbcUrl() == null || account.getJdbcUrl().isEmpty()) {
            return DriverManager.getConnection(this.jdbcUrl, account.getUser(), account.getPassword());
        } else {
            return DriverManager.getConnection(account.getJdbcUrl(), account.getUser(), account.getPassword());
        }
    }

}
